package cabler;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;

/**
 *	This program takes two arguments: a float (which is for alpha) and file name.
 */
public class FindBisection {
	float min=Integer.MAX_VALUE;
	static int arSize;     
	static Validator val;
	int[] copymin=new int[arSize];
	int global=0;
	int local=0;
	int cuts=0;
	ArrayList<int[]> LocMin=new ArrayList<int[]>();
	int numDone = 0;  //for progress reporting
	
	/** 
	 *	Enumerates total amount of n choose k of possibilities.
	 */
	public void enumerate(int n, int k, int[] list, Validator val, float alpha) {
		if (k==0) {
			HashSet<Integer> set=new HashSet<Integer>();
			//set of routers in one half
			for(int tmp : list)
				set.add(tmp);

			int totalGlobalCuts=0;
			// find total global cuts
			for(Pair p : val.arr){
				//check whether p is cut (i.e. exactly one endpoint is in the set)
    			int end1 = p.g1*val.numOfRouter+p.r1;
				int end2 = p.g2*val.numOfRouter+p.r2;
				if(set.contains(end1) != set.contains(end2))
					totalGlobalCuts++;
			}
			// find total local cuts
			int totalLocalCuts=findLocalCuts(set);
			
			// get overall cuts
			float totalcuts=totalGlobalCuts*alpha+totalLocalCuts;
			if((totalcuts < min) ||
			   ((totalcuts==min) && (totalGlobalCuts < global))){
			   //2nd condition so we minimize global cuts even when alpha=0
				min = totalcuts;
				for(int i=0;i<list.length;i++)
					this.copymin[i]=list[i];
				global=totalGlobalCuts;
				local =totalLocalCuts;
			}
		}else{
			if(n == k) {
				// only possibility is for the list to include n
				list[k-1] = n-1;
				enumerate(n-1, k-1, list, val, alpha);
			} else {
				// enumerate possibilities which include n
				list[k-1] = n-1;
				enumerate(n-1, k-1, list, val, alpha);

				// enumerate possibilities that do not include n
				enumerate(n-1, k, list, val, alpha);
			}
		}
	}

	/** 
	 *	Finds the number of local cuts 
	 */
    public int findLocalCuts(HashSet<Integer> set){
		int totalLocalCuts=0;
		int thisSide=0;
		int numOtherSide=0;
		//count the number of routers in the same set
		for (int x=0;x<val.numOfGroup;x++){
			for(int y=0;y<val.numOfRouter;y++){
				if (set.contains(x*val.numOfRouter+y))
					thisSide++;
			}
			//calculate #of local cuts based on numRouters in set

			numOtherSide=val.numOfRouter-thisSide;//find num of switches of other side
			totalLocalCuts+=thisSide*numOtherSide;

			thisSide=0;
		}
		return totalLocalCuts;
	}
	
	public int findGlobalCuts(Validator val, HashSet<Integer> set){
		int totalGlobalCuts=0;
		for(Pair p : val.arr){
			//check whether p is cut (i.e. exactly one endpoint is in the set)
			int end1 = p.g1*val.numOfRouter+p.r1;
			int end2 = p.g2*val.numOfRouter+p.r2;
			if(set.contains(end1) != set.contains(end2))
				totalGlobalCuts++;
		}
		return totalGlobalCuts;
	}
	
	/**
	 *	arg[0] alpha
	 *	arg[1] readfile name
	 */
	public static void main(String[]args) throws IOException{
		long startTime = System.currentTimeMillis(); // time the program
		
		BufferedReader reader = null;
		float alpha = -1;
		try {
			alpha = Float.parseFloat(args[0]);
			reader = new BufferedReader(new FileReader(args[1]));
		} catch (Exception e) {
			System.err.println("Failed to read program args: " + e.getMessage());
			System.exit(0);
		}
		val=new Validator(reader);	

		arSize=(val.numOfRouter * val.numOfGroup)/2;
		int[] ar = new int[arSize];
		int n = (val.numOfRouter * val.numOfGroup);  //total # switches
		FindBisection b = new FindBisection();	

		b.enumerate(n-1, arSize, ar, val, alpha);  
		//n-1 because assuming WLOG last switch is on "other" side"

		System.out.println(Arrays.toString(b.copymin));
		System.out.println("Minimum Overall Cost: "+b.min);
		System.out.println("Global edges: "+b.global +
				   " Local edges: "+b.local);
		System.out.println("Min cut nodes:");
		for(int i : b.copymin)
			System.out.printf("%d,",i);
		System.out.println();

		System.out.println("Total runtime was " + ((float)(System.currentTimeMillis() - startTime) / (1000*60*60)) + " hours");
	}

}
