package cabler;

//Update 7/9/2015 includes the 1GCC validator which will take in the array list from Validator 
// and tell if you have exactly 1 GCC 
// Update 7/14/2015 update Validate method to run faster by keeping an array of switches
//Update 7/14/2015 update 1GCC to run faster
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;

public class Validator {
	boolean [][] visit;//2-D array for verification of 1GCC
	int [] CountSwitches;
	HashMap<Vertex,ArrayList<Vertex>> map=new HashMap<Vertex,ArrayList<Vertex>>();; // create the Hashmap for the recursive 1GCC function
	ArrayList<Pair> arr=new ArrayList<Pair>();
	int numOfRouter,numOfGroup,GlobalEdgesPerRouter; 				//the size of the machine.
	

	public Validator(BufferedReader read) throws IOException{

		String line = null;
		line=read.readLine(); 					//read in the first line separately.
		String[] size = line.split("\\s");
		numOfRouter=Integer.parseInt(size[0]);
		numOfGroup=Integer.parseInt(size[1]);
		GlobalEdgesPerRouter=(numOfGroup - 1)/numOfRouter;
		visit= new boolean [numOfGroup][numOfRouter]; // 2-D array to for group/switch combination
		CountSwitches= new int [(numOfGroup*numOfRouter)];// array of switches for Validate method
		while ((line = read.readLine()) != null) {
			// read into the array list.
			String[] parts = line.split("\\s");

			arr.add(new Pair(Integer.parseInt(parts[0]),
					Integer.parseInt(parts[1]),
					Integer.parseInt(parts[2]),
					Integer.parseInt(parts[3])));
		}
		// Used to initialize the 2-D array all to False and to add required Vertexes to the ArrayList exists	
		/*for(int i=0;i<5;i++){//groups
			for(int j=0;j<2;j++){//switches
				visit[i][j]=false;
				//vertex.add(new Vertex (i, j));
			}
		}*/
			
	}
	/**verify that one router doesn't connect to another one in the same group 
	 * as well as it does the correct amount of connections. **/
	public boolean validate(ArrayList<Pair> arr,int numR,int numG){
		//int count=0;
		//int tmpG=-1;
		//int tmpR=-1;
		for (Pair p : arr){
			//verify it doesn't go to the same group 
			if (p.g1==p.g2){
				System.out.println("Go to the same group: "+p.g1+p.r1+p.g2+p.r2);
				return false;
			}
			else{// that specific pair connects somewhere so need to update the switch count
				CountSwitches[(p.g1*numOfRouter)+(p.r1)]=CountSwitches[(p.g1*numOfRouter)+(p.r1)]+1;
				CountSwitches[(p.g2*numOfRouter)+(p.r2)]=CountSwitches[(p.g2*numOfRouter)+(p.r2)]+1;
			}
		}
		for(int i=0;i<CountSwitches.length;i++){
			if(CountSwitches[i]>GlobalEdgesPerRouter){
				System.out.println("ExceededConnection at switch "+ CountSwitches[i]);
				return false;
			}
		}	
		return true;
	}
		/*for (Pair  p2 : arr){
				//count the occurrence of every router.
				if (p.g1==p2.g1&&p.r1==p2.r1
				  ||p.g1==p2.g2&&p.r1==p2.r2)
					count++;
			}
			if(count>((numG-1)/numR)){
				System.out.println("Exceeded connection "+count+" "+p.g1+p.r1+p.g2+p.r2);
				return false;
			}else 
				count=0;
		}*/
			
	public boolean Val1GCC(ArrayList<Pair> arr,boolean [][] visit){
		// Used to initialize the 2-D array which will tell if a specific switch was visited or not, needs to be initalized to false at the beginning
		for(int i=0;i<numOfGroup;i++){//groups 
			for(int j=0;j<numOfRouter;j++){//switches
				visit[i][j]=false;
			}
		}
		//create an ArrayList to iterate over the connections for the switch
		// go through each pair in the array arr
		// somehow the map doesn't see the values that were put there before key set is null and values is null can this be due to the fact 
		for(Pair n:arr){
			// split up the pairs
			int group1=n.get(0);
			int router1=n.get(1);
			int group2=n.get(2);
			int router2=n.get(3);
			// convert the pairs to the vertexes for the hashmap ver1 will be used as a key and ver2 will be used to represent all the nighbors 
			Vertex ver1=new Vertex(group1,router1);
			Vertex ver2=new Vertex(group2,router2);
			// put all of the connections into a hashmap
			//check if there is already this switch in a map and if it does...
			if(map.containsKey(ver1)){
				// get the values or the list of connections that switch has from the map
				ArrayList<Vertex> ListOfNeighbors;// will creating a new array delete everything that was stored there previously?
				ListOfNeighbors=map.get(ver1);
				// add a new neighbor to the list
				ListOfNeighbors.add(ver2); 
			}
			// need to add the edges both ways so now see if need to do the same thing in reverse
			if(map.containsKey(ver2)){
				// get the list of neighbors
				ArrayList<Vertex> ListOfNeighbors;
				ListOfNeighbors=map.get(ver2);
				// add the neighbor
				ListOfNeighbors.add(ver1);
			}
			// this switch doesn't have any connections yet
			if(!map.containsKey(ver1)) {
				ArrayList<Vertex> ListOfNeighbors= new ArrayList<Vertex>();
				// add a connection to the arraylist
				ListOfNeighbors.add(ver2);
				// add the switch and it's neighbor to the switch
				map.put(ver1,ListOfNeighbors);
				// check if other way vertex needs to be added
			}
			if(!map.containsKey(ver2)){
				ArrayList<Vertex> ListOfNeighbors= new ArrayList<Vertex>();
				ListOfNeighbors.add(ver1);
				map.put(ver2, ListOfNeighbors);
			}
		}

		Vertex InitialVertex= new Vertex (0,0);// pick an initial vertex to start looking
		Recursive1GCC(map,InitialVertex,visit);//call the recursive function
		for(int i=0;i<numOfGroup;i++){//go and check if there are unvisited switches which would be marked false
			for(int j=0;j<numOfRouter;j++){
				if(!visit[i][j]){
					return false;
				}
			}
		}
		return true;
	}

// Recursive method to check the switch and see if there is 1 GCC it's called by Val1GCC
	public void Recursive1GCC(HashMap<Vertex,ArrayList<Vertex>> map, Vertex ver, boolean [][] visit){
		int vertexGr=ver.getGroup(); // get group number of the vertexes
		int vertexSw=ver.getSwitch();// get switch number
		// mark the current switch as visited
		visit[vertexGr][vertexSw]=true;
		ArrayList<Vertex> Neighbors;
		// get the neighbors
		Neighbors= map.get(ver);
		if(Neighbors == null)
			System.err.println("Vertex " + ver + " has no neighbors");
		//iterate over the neighbors of ver
		for(Vertex l:Neighbors){
			int Group2=l.getGroup(); // get the neighbor's group
			int Switch2=l.getSwitch(); // get the neighbor's switch
			// check if this switch was visited and if it wasn't...
			if(!visit[Group2][Switch2]){
				//Vertex verNext= new Vertex(Group2,Switch2);
				// recurse on that switch
				Recursive1GCC(map, l, visit);	
			}
		}
	}
	
	public static void main(String[]args) throws IOException {
		BufferedReader reader = null;
		try {
			//The file path should be the local directory in the work space. 
			String filename;
			filename=args[0];
			reader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		//create one instance.
		Validator val=new Validator(reader);
		//Vertex InitialVertex=new Vertex(0,0);

		/**test if the machine cabling is valid and test if there is 1GCC in the whole connection scheme*/
		if (val.validate(val.arr, val.numOfRouter, val.numOfGroup)){
			System.out.println("Cable Verified");
			if (val.Val1GCC(val.arr,val.visit)) {
				System.out.println("1GCC Verified");
			} else  {
				System.out.println("1GCC NOT Verified");
			} 
		} else {
			System.out.println("Invalid Cabling");
		}
	}
}
