package dragonfly.graph;

public class Edge implements Comparable<Edge> {
	private Node source;
	private Node dest;
	private float cost;

	public String toString() {
		return getSrc().getName() + " => " + getDst().getName() + ": "
				+ getWeight();
	}
	
	/**
	 * Mathematica string
	 * @return
	 */
	public String mmaToString(){
		return getSrc().getName() + "<->" + getDst().getName();
	}

	public int hashCode() {
		return source.hashCode() * dest.hashCode() * (int) cost * 7;
	}

	public boolean equals(Object other) {
		if (!(other instanceof Edge)) {
			return false;
		}
		Edge e = (Edge) other;
		return cost == e.cost
				&& ((source == e.source && dest == e.dest) || (source == e.dest && dest == e.source));
	}

	public Node getSrc() {
		return this.source;
	}

	public Node getDst() {
		return this.dest;
	}

	public float getWeight() {
		return this.cost;
	}

	public Edge(Node s, Node d, float c) {
		this.source = s;
		this.dest = d;
		this.cost = c;
	}

	public int compareTo(Edge other) {
		if(this.cost > other.getWeight())
			return 1;
		else if (this.cost < other.getWeight())
			return -1;
		else 
			return 0;
	}
}
