package dragonfly.graph;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

public class Node {
	private String name;
	// map of other connected nodes and their costs
	private Map<Node, Float> edges = new HashMap<Node, Float>();

	// private Collection<Edge> edgeList=new HashSet<Edge>();

	public Node(String name) {
		this.name = name;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see INode#getName()
	 */
	public String getName() {
		return this.name;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see INode#getEdges()
	 */
	public Collection<Node> getNeighbors() {
		return edges.keySet();
	}

	private Map<Node, Float> getEdgesAndCosts() {
		return edges;
	}

	public String toString() {
		return name;
	}

	public void addUndirectedEdgeToNode(Node n, float cost) {
		this.addDirectedEdgeToNode(n, cost);
		n.addDirectedEdgeToNode(this, cost);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see graphs.INode#addDirectedEdgeToNode(graphs.INode, int)
	 */
	public void addDirectedEdgeToNode(Node n, float weight) {
		this.edges.put(n, weight);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see graphs.INode#removeUndirectedEdgeToNode(graphs.INode)
	 */
	public void removeUndirectedEdgeToNode(Node n) {
		this.removeUndirectedEdgeToNode(n);
		n.removeDirectedEdgeToNode(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see graphs.INode#removeDirectedEdgeToNode(graphs.INode)
	 */
	public void removeDirectedEdgeToNode(Node n) {
		this.edges.remove(n);
	}

	/*
	 * (non-Javadoc)
	 * don't call this function on females
	 * @see graphs.INode#getWeight(graphs.INode)
	 */
	public float getWeight(Node n) {
		if (!hasEdge(n)) {
			throw new IllegalStateException("No edge from " + getName()
					+ " to " + n.getName());
		}
		return edges.get(n);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see graphs.INode#hasEdge(graphs.INode)
	 */
	public boolean hasEdge(Node other) {
		return edges.containsKey(other);
	}

	Node getRandomNeighbor() {
		Random r = new Random();
		int stop = r.nextInt(edges.size());
		int i = 0;
		for (Node n : edges.keySet()) {
			if (i == stop) {
				return n;
			}
			i++;
		}
		return null;
	}
	
	public ArrayList<Edge> getAllEdges(){
		ArrayList<Edge> edges = new ArrayList<Edge>();
		for(Node n : this.getNeighbors())
			edges.add(new Edge(this,n, getWeight(n)));
		return edges;
		
	}
}
