#!/usr/bin/perl -w

#finds bisection bandwidth as a function of alpha

#assumes presence of directories runs, results, and done
#runs gets the output of calls to java program that finds bisection bandwidth
#results gets a summary for each file
#done gets link description files that are complete

#puts warnings and errors into file "messages"

#takes name of link description file from command line (assumed to be in working directory)

$log = 'messages';

sub writeout($$) {
    my $file = shift @_;
    my $mesg = shift @_;
    open(OUT, ">> $file") or die;
    print OUT "$mesg\n";
    close(OUT);
}

sub error($) {
    my $mesg = shift @_;
    writeout($log, "ERROR: $mesg");
    exit;
}

$numRuns = 0;   #number of times we've run the java code (bails if too many)

sub testrun($$) {
    my $file = shift @_;
    my $alpha = shift @_;

    print "\nGive global and local edges for alpha=$alpha\n";
    my $glo = <STDIN>;
    my $loc = <STDIN>;
    chop $glo;
    chop $loc;
    return ($glo,$loc);
}

sub run($$) {
    my $file = shift @_;
    my $alpha = shift @_;
    my $outfile = "runs/$file-$alpha";
    my $res = '';
    $numRuns++;
    if($numRuns == 10) {
	writeout($log, "stopped $file on 10th run");
	exit;
    }
    if(open(OUTFILE, "< $outfile")) {  #use previous run if it exists
	while($line = <OUTFILE>) {
	    $res = $line if($line =~ /Global edges/);
	}
	close(OUTFILE);
    }
    if($res eq '') {
	writeout($log, "calling for $outfile");
	$res =`java cabler/FindBisection $alpha $file`;
	writeout($outfile, $res);
    } else {
	writeout($log, "successfully read previous run from $outfile");
    }
    $res =~ /Global edges: ([0-9]+) Local edges: ([0-9]+)/ or error("match failure in $outfile");
    return ($1, $2);
}

sub findBetween($$$$$);   #prototype to avoid warning in recursive calls
sub findBetween($$$$$) {
    my $file = shift @_;
    my $lowGlo = shift @_;
    my $lowLoc = shift @_;
    my $hiGlo = shift @_;
    my $hiLoc = shift @_;

    #y=mx+b
    #total = global_1*alpha + local_1 = global_2*alpha + local_2
    #solves to alpha = (local_1-local_2)/(global_2-global_1)
    my $alpha = ($lowLoc - $hiLoc) / ($hiGlo - $lowGlo);
    my @res = run($file, $alpha);
    my $midGlo = shift @res;
    my $midLoc = shift @res;

    #either the lines meet in the middle or the value at the midpoint is less,
    #in which case we need to recurse on both sides
    if($lowGlo*$alpha+$lowLoc != $midGlo*$alpha+$midLoc) {
	findBetween($file, $lowGlo, $lowLoc, $midGlo, $midLoc);
	findBetween($file, $midGlo, $midLoc, $hiGlo, $hiLoc);
    } else {
	writeout($resfile, "$alpha\t$hiGlo\t$hiLoc");
    }
}

#"main" code:
$arg = shift @ARGV;
$resfile = "results/$arg.res";
@res = run($arg, 0);
$glo_0 = shift @res;
$loc_0 = shift @res;
@res = run($arg, 100);
$glo_100 = shift @res;
$loc_100 = shift @res;
writeout($log, "WARNING: global edges in cut at alpha=100 for $arg") if($glo_100 != 0);
writeout($resfile, "0\t$glo_0\t$loc_0");
if(($glo_0 == $glo_100) and ($loc_0 == $loc_100)) {
} else {
    findBetween($arg, $glo_0, $loc_0, $glo_100, $loc_100);
}
writeout($log, "finished $arg");
system "mv $arg done/$arg";
